@@ -1,5 +1,5 @@
 package Mail::DMARC;
-our $VERSION = '1.20150310'; # VERSION
+our $VERSION = '1.20150317'; # VERSION
 use strict;
 use warnings;
 
@@ -192,6 +192,44 @@ sub is_subdomain {
     return $_[0]->{is_subdomain} = $_[1];
 }
 
+sub get_report_window {
+    my ( $self, $interval, $now ) = @_;
+
+    my $min_interval = $self->config->{'report_sending'}{'min_interval'};
+    my $max_interval = $self->config->{'report_sending'}{'max_interval'};
+
+    $interval = 86400 if ! $interval; # Default to 1 day
+    if ( $min_interval ) {
+        $interval = $min_interval if $interval < $min_interval;
+    }
+    if ( $max_interval ) {
+        $interval = $max_interval if $interval > $max_interval;
+    }
+
+    if ( ( 86400 % $interval ) != 0 ) {
+        # Interval does not fit into a day nicely,
+        # So don't work out a window, just run with it.
+        return ( $now, $now + $interval - 1);
+    }
+
+    my $begin = $self->get_start_of_zulu_day( $now );
+    my $end = $begin + $interval - 1;
+
+    while ( $end < $now ) {
+        $begin = $begin + $interval;
+        $end   = $begin + $interval - 1;
+    }
+
+    return ( $begin, $end );
+}
+
+
+sub get_start_of_zulu_day {
+    my ( $self, $t ) = @_;
+    my $start_of_zulu_day = $t - ( $t % 86400 );
+    return $start_of_zulu_day;
+}
+
 sub save_aggregate {
     my ($self) = @_;
 
@@ -201,8 +239,11 @@ sub save_aggregate {
     foreach my $f ( qw/ org_name email extra_contact_info report_id / ) {
         $agg->metadata->$f( $self->config->{organization}{$f} );
     };
-    $agg->metadata->begin( time );
-    $agg->metadata->end( time + ($self->result->published->ri || 86400 ));
+
+    my ( $begin, $end ) = $self->get_report_window( $self->result->published->ri, time() );
+
+    $agg->metadata->begin( $begin );
+    $agg->metadata->end( $end );
 
     $agg->policy_published( $self->result->published );
 
@@ -249,7 +290,7 @@ Mail::DMARC - Perl implementation of DMARC
 
 =head1 VERSION
 
-version 1.20150310
+version 1.20150317
 
 =head1 SYNOPSIS
 
@@ -573,7 +614,7 @@ Davide Migliavacca <shari@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Benny Pedersen ColocateUSA.net Marc Bradshaw Ricardo Signes
+=for stopwords Benny Pedersen Marc Bradshaw Ricardo Signes
 
 =over 4
 
@@ -583,10 +624,6 @@ Benny Pedersen <me@junc.eu>
 
 =item *
 
-ColocateUSA.net <company@colocateusa.net>
-
-=item *
-
 Marc Bradshaw <marc@marcbradshaw.net>
 
 =item *
